__version__ = '0.2.2'

from .exceptions import (
    SourcesException, BadResponseException, UnknownSourceException,
    UnavailableException, ChapterException,
)

from .models import Novel, Volume, Chapter, Metadata

from .sources import (
    Source, MetaSource,
    sources, meta_sources
)

from .utils import (
    parse_source, parse_metasource,
    novel_source_types, metadata_source_types,
    locate_novel_source, locate_metadata_source,
)



import importlib
import inspect
from abc import ABC
from functools import lru_cache
from pathlib import Path
from typing import List, TypeVar, Type

from deprecation import deprecated

from novelsave_sources.exceptions import UnknownSourceException
from novelsave_sources.sources import sources, meta_sources
from . import __version__
from .sources.metadata.metasource import MetaSource
from .sources.novel.source import Source


@deprecated(deprecated_in="0.2.2", removed_in="0.3.0", current_version=__version__,
            details="Use 'locate_novel_source' function instead")
def parse_source(url):
    """Locate and create a source parser for the url if schema exists

    :throws UnknownSourceException: if the url cannot be parsed by any existing source schema
    """
    for source in sources:
        if source.of(url):
            return source()

    raise UnknownSourceException(f'"{url}" does not correspond to any available source')


@deprecated(deprecated_in="0.2.2", removed_in="0.3.0", current_version=__version__,
            details="Use 'locate_metadata_source' function instead")
def parse_metasource(url: str):
    """Locate and create a source parser for the url if schema exists

    :throws UnknownSourceException: if the url cannot be parsed by any existing source schema
    """
    for source in meta_sources:
        if source.of(url):
            return source()

    raise UnknownSourceException(f'"{url}" does not correspond to any available metadata source')


_T = TypeVar('_T')


@lru_cache()
def _find_impl(r_location: str, interface: Type[_T]) -> List[Type[_T]]:
    """Finds the implementations of the interface in the provided package location"""
    impls = []
    package = r_location.replace('../', '.').replace('/', '.').rstrip('.')
    for path in (Path(__file__) / r_location).glob('*.py'):
        if not path.is_file():
            continue
        elif path.name.startswith('_'):  # private files
            continue

        module = importlib.import_module(f'{package}.{path.stem}', 'novelsave_sources')
        for name, member in inspect.getmembers(module, inspect.isclass):
            # the member/class must be a subclass of interface and should be an implementation
            if issubclass(member, interface) and member is not interface and not isinstance(member, ABC):
                impls.append(member)

    return impls


def novel_source_types() -> List[Type[Source]]:
    """Locate and return all the novel source types"""
    return _find_impl('../sources/novel', Source)


def locate_novel_source(url: str) -> Type[Source]:
    """Locate and return the novel source parser for the url if it is supported

    :raises UnknownSourceException: if the url cannot be parsed by any existing source schema
    """
    for source in novel_source_types():
        if source.of(url):
            return source

    raise UnknownSourceException(f'"{url}" does not correspond to any available source')


def metadata_source_types() -> List[Type[MetaSource]]:
    """Locate and return all the metadata source types"""
    return _find_impl('../sources/metadata', MetaSource)


def locate_metadata_source(url: str) -> Type[MetaSource]:
    """Locate and return the metadata source parser for the url if it is supported

    :raises UnknownSourceException: if the url cannot be parsed by any existing source schema
    """
    for source in metadata_source_types():
        if source.of(url):
            return source

    raise UnknownSourceException(f'"{url}" does not correspond to any available metadata source')



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['novelsave_sources',
 'novelsave_sources.models',
 'novelsave_sources.sources',
 'novelsave_sources.sources.metadata',
 'novelsave_sources.sources.novel']

package_data = \
{'': ['*']}

install_requires = \
['beautifulsoup4>=4.10.0,<5.0.0',
 'cloudscraper>=1.2.58,<2.0.0',
 'deprecation>=2.1.0,<3.0.0',
 'lxml>=4.6.3,<5.0.0',
 'requests>=2.26.0,<3.0.0']

setup_kwargs = {
    'name': 'novelsave-sources',
    'version': '0.2.2',
    'description': 'A collection of novel sources offering varying amounts of scraping capability.',
    'long_description': '# 📚 Novelsave Sources\n\nA collection of novel sources offering varying amounts of scraping capability.\n\n> Request a new source by [creating a new issue](https://github.com/mHaisham/novelsave_sources/issues/new/choose)\n\n## 📒 Sources\n\n### Novel\n\n#### ✅ Supported\n\n<table>\n    <thead>\n        <tr>\n            <th align="center">Lang</th>\n            <th>Source</th>\n            <th align="center">Search</th>\n            <th align="center">Last Checked</th>\n        </tr>\n    </thead>\n    <tbody>\n        <tr>\n            <td align="center">en</td>\n            <td>https://betwixtedbutterfly.com</td>\n            <td align="center"></td>\n            <td align="center">2021-09-03</td>\n        </tr>\n        <tr>\n            <td align="center">en</td>\n            <td>https://boxnovel.com</td>\n            <td align="center"></td>\n            <td align="center">2021-09-07</td>\n        </tr>\n        <tr>\n            <td align="center">en</td>\n            <td>https://chrysanthemumgarden.com/</td>\n            <td align="center"></td>\n            <td align="center">2021-09-07</td>\n        </tr>\n        <tr>\n            <td align="center">en</td>\n            <td>https://creativenovels.com</td>\n            <td align="center"></td>\n            <td align="center">2021-09-17</td>\n        </tr>\n        <tr>\n            <td align="center">en</td>\n            <td>https://dragontea.ink/</td>\n            <td align="center"></td>\n            <td align="center">2021-09-07</td>\n        </tr>\n        <tr>\n            <td align="center">en</td>\n            <td>https://dummynovels.com</td>\n            <td align="center"></td>\n            <td align="center">2021-09-07</td>\n        </tr>\n        <tr>\n            <td align="center">en</td>\n            <td>https://forums.spacebattles.com</td>\n            <td align="center"></td>\n            <td align="center">2021-09-09</td>\n        </tr>\n        <tr>\n            <td align="center">en</td>\n            <td>https://forums.sufficientvelocity.com</td>\n            <td align="center"></td>\n            <td align="center">2021-09-09</td>\n        </tr>\n        <tr>\n            <td align="center">en</td>\n            <td>https://novelfull.com</td>\n            <td align="center"></td>\n            <td align="center">2021-09-07</td>\n        </tr>\n        <tr>\n            <td align="center">en</td>\n            <td>https://novelfun.net</td>\n            <td align="center"></td>\n            <td align="center">2021-09-07</td>\n        </tr>\n        <tr>\n            <td align="center">en</td>\n            <td>https://novelgate.net</td>\n            <td align="center"></td>\n            <td align="center">2021-09-03</td>\n        </tr>\n        <tr>\n            <td align="center">en</td>\n            <td>https://novelonlinefull.com</td>\n            <td align="center"></td>\n            <td align="center">2021-09-07</td>\n        </tr>\n        <tr>\n            <td align="center">en</td>\n            <td>https://novelsite.net</td>\n            <td align="center"></td>\n            <td align="center">2021-09-06</td>\n        </tr>\n        <tr>\n            <td align="center">en</td>\n            <td>https://novelsrock.com</td>\n            <td align="center"></td>\n            <td align="center">2021-09-04</td>\n        </tr>\n        <tr>\n            <td align="center">en</td>\n            <td>https://peachpitting.com</td>\n            <td align="center"></td>\n            <td align="center">2021-09-04</td>\n        </tr>\n        <tr>\n            <td align="center">en</td>\n            <td>https://rainofsnow.com/</td>\n            <td align="center"></td>\n            <td align="center">2021-09-04</td>\n        </tr>\n        <tr>\n            <td align="center">en</td>\n            <td>https://readlightnovels.net</td>\n            <td align="center"></td>\n            <td align="center">2021-09-06</td>\n        </tr>\n        <tr>\n            <td align="center">en</td>\n            <td>https://wuxiaworld.online</td>\n            <td align="center"></td>\n            <td align="center">2021-09-07</td>\n        </tr>\n        <tr>\n            <td align="center">en</td>\n            <td>https://wuxiaworld.site</td>\n            <td align="center"></td>\n            <td align="center">2021-09-03</td>\n        </tr>\n        <tr>\n            <td align="center">en</td>\n            <td>https://www.chickengege.org</td>\n            <td align="center"></td>\n            <td align="center">2021-09-07</td>\n        </tr>\n        <tr>\n            <td align="center">en</td>\n            <td>https://www.foxaholic.com</td>\n            <td align="center"></td>\n            <td align="center">2021-09-03</td>\n        </tr>\n        <tr>\n            <td align="center">en</td>\n            <td>https://www.lightnovelworld.com</td>\n            <td align="center"></td>\n            <td align="center">2021-09-07</td>\n        </tr>\n        <tr>\n            <td align="center">en</td>\n            <td>https://www.mtlnovel.com</td>\n            <td align="center"></td>\n            <td align="center">2021-09-07</td>\n        </tr>\n        <tr>\n            <td align="center">en</td>\n            <td>https://www.novelhall.com</td>\n            <td align="center"></td>\n            <td align="center">2021-09-07</td>\n        </tr>\n        <tr>\n            <td align="center">en</td>\n            <td>https://www.novelpassion.com</td>\n            <td align="center"></td>\n            <td align="center">2021-09-04</td>\n        </tr>\n        <tr>\n            <td align="center">en</td>\n            <td>https://www.readlightnovel.me</td>\n            <td align="center"></td>\n            <td align="center">2021-09-07</td>\n        </tr>\n        <tr>\n            <td align="center">en</td>\n            <td>https://www.royalroad.com</td>\n            <td align="center"></td>\n            <td align="center">2021-09-06</td>\n        </tr>\n        <tr>\n            <td align="center">en</td>\n            <td>https://www.scribblehub.com</td>\n            <td align="center"></td>\n            <td align="center">2021-09-06</td>\n        </tr>\n        <tr>\n            <td align="center">en</td>\n            <td>https://www.wattpad.com</td>\n            <td align="center"></td>\n            <td align="center">2021-09-06</td>\n        </tr>\n        <tr>\n            <td align="center">en</td>\n            <td>https://www.webnovel.com</td>\n            <td align="center"></td>\n            <td align="center">2021-09-03</td>\n        </tr>\n        <tr>\n            <td align="center">en</td>\n            <td>https://www.wuxiaworld.co</td>\n            <td align="center"></td>\n            <td align="center">2021-09-04</td>\n        </tr>\n        <tr>\n            <td align="center">en</td>\n            <td>https://www.wuxiaworld.com</td>\n            <td align="center"></td>\n            <td align="center">2021-09-04</td>\n        </tr>\n    </tbody>\n</table>\n\n#### ❌ Rejected\n\n<table>\n    <thead>\n        <tr>\n            <th align="center">Lang</th>\n            <th>Source</th>\n            <th>Reason</th>\n        </tr>\n    </thead>\n    <tbody>\n        <tr>\n            <td align="center">en</td>\n            <td>https://www.fanfiction.net</td>\n            <td>Has cloudflare bot protection</td>\n        </tr>\n    </tbody>\n</table>\n\n### Metadata\n\n#### ✅ Supported\n\n<table>\n    <thead>\n        <tr>\n            <th align="center">Lang</th>\n            <th>Metadata Source</th>\n            <th align="center">Last Checked</th>\n        </tr>\n    </thead>\n    <tbody>\n        <tr>\n            <td align="center">en</td>\n            <td>https://www.novelupdates.com</td>\n            <td align="center">2021-09-03</td>\n        </tr>\n        <tr>\n            <td align="center">en</td>\n            <td>https://www.wlnupdates.com/</td>\n            <td align="center">2021-08-25</td>\n        </tr>\n    </tbody>\n</table>\n\n## 📝 Disclaimer\n\nWe are not affiliated, associated, authorized, endorsed by, or in any way officially\nconnected with any of the [sources](#sources) mentioned above.\n\n## 📜 License\n\n[Apache-2.0](https://github.com/mHaisham/novelsave_sources/blob/master/LICENSE)\n',
    'author': 'Mensch272',
    'author_email': '47662901+mensch272@users.noreply.github.com',
    'maintainer': 'Mensch272',
    'maintainer_email': '47662901+mensch272@users.noreply.github.com',
    'url': 'https://github.com/mensch272/novelsave_sources',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
